

/*****************************************************************************

ScrumPy -- Metabolic Modelling with Python

Copyright Mark Poolman 1995 - 2002

 This file is part of ScrumPy.

    ScrumPy is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    ScrumPy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ScrumPy; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


*****************************************************************************/
/********

M.G. Poolman 11/5/2001

LowLevelKinMod.c -- C++ class interface to Scampi model API,
to act as a "shadow class" for python via swig

********/


    /** standard includes **/
#include <string.h>
#include <dlfcn.h>
#include <stdlib.h>
#include <fstream>
#include <math.h>

    /** local includes  **/


#include <ScampiModelBase.h>
#include <Scampi.h>
#include <HandyMacros.h>


#include "LowLevelKinMod.h"



/*

#include <signal.h>
void Abort(int NotUsed){

  char c = 0 ;

    while (1){
        cerr << "caught ^C, enter : \na - abort\ne - exit\nc - continue\n? " ;
        cin >> c ;
        switch(c){
            case 'a': abort()  ;
            case 'e': exit(-1) ;
            case 'c': return ;
        }
    }
}
*/

double NottaNum(void){ return sqrt(-1.0) ;} ;



#define REALSIZE sizeof(double)





void LowLevelKinMod::Init(void){


    /*signal(SIGINT, Abort) ;*/
    cerr << "LLKM cons" SP this NL ;
    mod = 0 ;
    dlib = 0 ;
    dlibname = 0 ;
    err = Undefined ;

    n_Refs = 1 ;

    ConcBuf = RescueConcs = 0 ;
    SSTol       = DefaultSSTol ;
    NewtIts     = DefaultNewtIts ;
    SimTol      = DefaultSimTol ;
    Tincr       = DefaultTincr ;
    Time        = 0 ;
    ios::sync_with_stdio() ; // make sure Scampi stderr syncs w/ cerr
} ;


LowLevelKinMod::LowLevelKinMod() {
    cerr << "\n14/11/05  !!!!!!!\n" ;
    Init() ;
} ;


void LowLevelKinMod::Destroy(){

  char *delstr ;
    cerr << "LowLevelKinMod::Destroy()102020\n" ;
    if (dlib != 0){
        cerr << "dlib" SP dlib NL ;
        dlclose(dlib) ;
        delstr = Cat("rm -f ", dlibname) ;
        cerr << "LowLevelKinMod::Destroy()" SP delstr NL ;
        system(delstr) ;
        free(delstr) ;
        mod = 0 ;
    }
    cerr << "VarTab.Clear() ? " ;
    VarTab.Clear() ;
    cerr << "OK\n" ;
    mod = 0 ;
    dlib = 0 ;
    cerr << "free(dlibname)? " ;
    if (dlibname){
        free(dlibname) ;
        dlibname = 0 ;
    }
    cerr << "OK\n" ;

}

LowLevelKinMod::~LowLevelKinMod(){

    cerr << "\nLLKM dest\n\n" SP dlib ;
    Destroy() ;

} ;

bool LowLevelKinMod::Usable() {
      /* pre: true
        post: Usable <=> model is usable */

    return (mod != 0) ;
} ;



void LowLevelKinMod::Load(const char *modname, const char *libname){
      /* pre: true
        post: Attempts to load model from file name. Hints to stderr.
              User should now test model with Usable(). ErrorMsg() will
              return a suitable message */

    int n ;


    if(mod != 0)
        cerr << "\n!! Reloading LowLevelKinMod = mem leak !!\n" ;
    mod = NewMD(modname, libname, &dlib) ;
    cerr << "LLKM dlib = " SP dlib NL ;
    if (mod == 0)
        err = Undefined ;
    else{
        err = OK ;
        dlibname = Copy(libname) ;

        Time = mod->Time ;

        for(n = 0 ; n < mod->n_Params ; n++)
            VarTab.Add(mod->Param_Names[n], mod->Param_Vals + n, Param) ;

        for(n = 0 ; n < mod->n_Mets ; n++)
            VarTab.Add(mod->Met_Names[n], mod->Met_Vals + n, Conc) ;

        for(n = 0 ; n < mod->n_Reacts ; n++)
            VarTab.Add(mod->React_Names[n], mod->React_Rates + n, Vel) ;

        VarTab.Add(TimeString, Time, ::Time) ;

        Time = mod->Time ;

        if(RescueConcs != 0)
            delete[] RescueConcs ;

        RescueConcs = new double[mod->n_Mets] ;
        ConcBuf = new double[mod->n_Mets] ;
        memcpy(RescueConcs, mod->Met_Vals, mod->n_Mets * REALSIZE) ;



        GetRescue() ; // Renew does initial eval, which we don't want
        Eval() ;

        err = OK ;

    }
} ;

bool LowLevelKinMod::Exists(const char *name) {
      /* pre: Usable() ;
        post: Exists(name) <=> this has a quantity associated with name */

  return(VarTab.Search(name) != 0) ;
}



double LowLevelKinMod::GetVal(const char *name) {
      /* pre: Exists(name)
        post: GetVal(name)  == value associated with name */


    double *targ ;

    targ = VarTab.Search(name) ;

    if(!targ){
        cerr << "!! Warning attempted to get value of non-existent name" SP name
                SP "in LowLevelKinMod" SP mod->Name NL ;
        return(NottaNum()) ;
    }
    return(*targ) ;
} ;



double *LowLevelKinMod::GetVal_p(const char *name) {
      /* pre: Exists(name)
        post: GetVal_p(name)  == pointer tp value associated with name */

    return VarTab.Search(name) ;
} ;


/*enum Component_id int for swig/python */
int  LowLevelKinMod::GetId(const char *name){
      /* pre: Exists(Name) ;
        post: GetId(Name) == id of name */

    return (int) VarTab.GetId(name) ;
};


void LowLevelKinMod::SetVal(const char *name, double val) {
        /* pre: Exists(name)
          post: GetVal(name)  == val */

    double *targ ;
    targ = VarTab.Search(name) ;

    if(!targ)
        cerr << "!! Warning attempted to set value of non-existent name" SP name
                SP "in LowLevelKinMod" SP mod->Name NL ;
    else{
        *targ = val ;
        // cerr << "no ri in sv\n" ;
        // ResetInteg() ;
    }
}




int LowLevelKinMod::SizeVec(enum Component_id comp) {
        /* pre: Usable()
          post: SizeVec(comp) == size of vector associated with comp */

    switch(comp){
        case Param: return(mod->n_Params) ;
        case Conc:  return(mod->n_Mets) ;
        case Vel:   return(mod->n_Reacts) ;
        case dMet:  return(mod->n_Mets) ;
        case ::Time:    return 1 ;
    }
    cerr << "!!!! LLKM: Bad comp val" SP comp SP "in SizeVec()," ;
    cerr << " hit ctrl+C and call the programmer !!!!" NL ;
    return (-1) ;
} ;



int LowLevelKinMod::n_Cons(void) {
    /* pre: Usable() ;
          post: n_Cons() == number of conserved metabolites */
    return(mod->n_Cons) ;
} ;


void LowLevelKinMod::UpdateCons(void){
        /* pre: Usable()
          post: Concentrations updated to reflect current csum params */

    int  n ;

    for(n = 0 ; n < mod->n_Cons ; n++)
        mod->Met_Vals[n+ mod->n_Free] = mod->Met_Cons_Eqns[n]() ;
} ;


double *LowLevelKinMod::GetVec(enum Component_id comp) {
        /* pre: Usable()
          post: GetVec(comp) = (pointer to first element in) vector associated
                with comp */


    switch(comp){
        case Param: return(mod->Param_Vals) ;
        case Conc:  return(mod->Met_Vals) ;
        case Vel:   return(mod->React_Rates) ;
        case dMet:  return(mod->dMet_dT) ;

    }

    cerr << "!!!! Bad comp val " SP comp SP "in GetVec()," ;
    cerr << " hit ctrl+C and call the programmer !!!!" NL ;
    return (0) ;
} ;




double *LowLevelKinMod::CpyVec(enum Component_id comp){
        /* pre: Usable()
          post: returns a new copy of GetVec(comp), which it is the user's
                responsobility do delete */

    double *v = GetVec(comp) ;
    int  len = SizeVec(comp) ;
    double *rv = new double[len] ;

     memcpy(rv, v, len * sizeof(double)) ;
     return rv ;
} ;


void LowLevelKinMod::NaNVec(enum Component_id comp) {
        /* pre: Usable()
          post: All els of vec are  NaN */

    cerr << "NaNVec\n" ;
    double *v = GetVec(comp) ;
    int  len = SizeVec(comp), n ;

    for(n=0 ; n < len ; n++)
        v[n] = NottaNum() ;

    cerr << "NaNVecDone\n" ;
} ;


void LowLevelKinMod::ZerVec(enum Component_id comp) {
        /* pre: Usable()
          post: All els of vec are zero */

    cerr << "ZerVec\n" ;
    double *v = GetVec(comp) ;
    int  len = SizeVec(comp), n ;

    for(n=0 ; n < len ; n++)
        v[n] = 0.0 ;
    cerr << "ZerVecDone\n" ;
} ;


void LowLevelKinMod::SetVec(enum Component_id comp, double *vec) {
        /* pre: Usable()
          post: vec is copied into vector associated with comp */

    /***** Never seen by Python, see c2py/PysetVev()  *****/

    memcpy(GetVec(comp), vec, SizeVec(comp) * REALSIZE) ;
    ResetInteg() ;

} ;


const char **LowLevelKinMod::GetNameVec(enum Component_id comp){


    switch(comp){
        case Param: return(mod->Param_Names) ;
        case Conc:  return(mod->Met_Names) ;
        case Vel:   return(mod->React_Names) ;
        // case ::Time: return((const char **)&TimeString) ;
    }

    cerr << "\n!!!\tBad Component_id = ", (int) comp,
            " in LowLevelKinMod::GetNameVec() - hit ^C and call the programmer !!\n" ;

    return 0 ;
} ;



void LowLevelKinMod::SetRescue(void){
        /* pre: Usable()
          post: Rescue concentrations set to current concentrations */

    UpdateCons() ;
    memcpy(RescueConcs, GetVec(Conc), mod->n_Mets * REALSIZE) ;
} ;


void LowLevelKinMod::GetRescue(void){
        /* pre: Usable()
          post: Current concentrations set to rescue concentrations */

    memcpy(GetVec(Conc), RescueConcs,  mod->n_Mets * REALSIZE) ;
    ResetInteg() ;
    /* TODO: should have a ResetSS() here as well, not essential at this time Aug/2010,
             as current SS sp is much simpler than the Integ sp */
} ;

                                    /** the following all have pre: Usable() **/
void LowLevelKinMod::SetSimTol(double tol) {    // pre: tol > 0.0
    SimTol = tol ;
} ;


void LowLevelKinMod::SetTincr(double tincr) {   // pre: incr > 0.0
    Tincr = tincr ;
} ;

double LowLevelKinMod::GetTincr(void){
    return Tincr ;
} ;



void LowLevelKinMod::ResetInteg(void){
    ResetIntegrator(mod) ;
} ;


bool LowLevelKinMod::CSumsOK(void){
        /* pre: true ;
          post: CSumsOK() => Current free met values do not violate
                conservation relationships */

    int n ;
    for(n = 0 ; n < mod->n_Cons ; n ++)
        if( mod->Met_Cons_Eqns[n]() < 0.0)
            return false ;

    return true ;
} ;


bool LowLevelKinMod::IsOK(void){
        /* pre: true ;
          post: IsOK() => the last numerical action completed succesfully,
                ELSE velocity and concentration values not defined, and
                     should be checked +/- updated */

    return (err == OK) ;
} ;



const char *LowLevelKinMod::ErrorMsg(void){
        /* pre: true ;
          post: ErrorMsg() = Description of most recent error */

    return (SPI_errMsgs[err]) ;
} ;

int LowLevelKinMod::LastErr(void) {
        /* pre: true ;
          post: LastErr() == last error as defined in ScampiModelBase.h */
    return(int(err)) ;
} ;

void LowLevelKinMod::ClearErr(void){

    err = OK ;
} ;




void LowLevelKinMod::Eval(void){

    ModEval(mod) ;
} ;



void LowLevelKinMod::EvalNC(void){

    ModEvalNC(mod) ;
} ;

double LowLevelKinMod::GetSSErr(void){

  double sum = 0.0, ode ;
  int n ;

  for (n=0 ; n< mod->n_Free ; n++){
    ode =  mod->dMet_dT[n] ;
    sum += ode*ode ;
  }
  return sqrt(sum) ;
}




void LowLevelKinMod::CalcVels(void) {
    /* pre: true
      post: velocity vector ONLY updated (no calculation of conserved moieties or ODEs) */

      int n ;

      for(n =  0 ; n < mod->n_Reacts ; n++)
        mod->React_Rates[n] = mod->Rate_Eqns[n]() ;
} ;



void LowLevelKinMod::MinPackFCN(int *N, double *MetVals, double *dMet_dT, int *err){

  double *mod_MetVals = mod->Met_Vals, *mod_dMet_dT = mod->dMet_dT ;  // hang on to originals
  int n ;

    MetVals-- ;      // remove the fortran style 1 offset
    dMet_dT -- ;

    mod->Met_Vals = MetVals ;
    mod->dMet_dT = dMet_dT ;

    for(n = 0 ; n < mod->n_Cons ; n++)                      // calculate dependent conc values
        MetVals[mod->n_Free + n] = mod->Met_Cons_Eqns[n]() ;

    for(n =  0 ; n < mod->n_Reacts ; n++)                   // calculate reaction rates
        mod->React_Rates[n] = mod->Rate_Eqns[n]() ;

    for(n = 0 ; n < mod->n_Free ; n++)                 // calculate ODE vals
        dMet_dT[n] = mod->Met_ODEs[n]() ;

    mod->Met_Vals = mod_MetVals ;                            // restore original vectors
    mod->dMet_dT = mod_dMet_dT ;
}










void LowLevelKinMod::Simulate(int n_points){
        /* pre:  n_points > 0
          post: IsOk() => Succesful simulation to time + n_points *tincr, time updated
                ELSE problem with integration, possible hints on sterr */

  double tend ;

    err = OK ;
    for (int n = 0 ; n < n_points && err == OK ; n++){
         tend = *Time + Tincr ;

         // cerr << "Simulate" SP *Time SP Tincr SP tend NL ;
         OneShotSim(mod, tend, SimTol, &err) ;
         if(err != OK){
             ResetIntegrator(mod) ;
             OneShotSim(mod, tend, SimTol, &err) ;
             if(err != OK){
                 cerr << "simulate():\tbad point at t =" SP tend SP ErrorMsg() NL ;
                 ResetIntegrator(mod) ;
             }
         }
         *Time = tend ;
    }
} ;

void LowLevelKinMod::SimToTime(double tend){


    Tincr = tend - *Time ;
    Simulate(1) ;

} ;





double LowLevelKinMod::NewtSS(void){
          /* pre: Usable() ;
          post: Attempts raw Newton to get SS, returns tol achieved */

    double rv ;
    cerr << "LowLevelKinMod::NewtSS(void) Aug12 1535\n" ;
    rv = Find_ss(mod, NewtIts, SSTol, &err) ;
    if (err != OK){
        GetRescue() ;
        Eval() ;
        cerr << "LowLevelKinMod::NewtSS(void) !OK" SP err SP rv NL ;
    }
    cerr << "LowLevelKinMod::NewtSS(void) " SP err SP rv NL ;
    return rv ;
} ;













