

/*****************************************************************************

ScrumPy -- Metabolic Modelling with Python

Copyright Mark Poolman 1995 - 2002

 This file is part of ScrumPy.

    ScrumPy is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    ScrumPy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ScrumPy; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


*****************************************************************************/
#include<string.h>
#include<stdlib.h>
 

#include <Node.h>

#include "VarTbl.h"

class MVT_dat : public NodeUsrDat {

  public:


	MVT_dat() {
		Name = 0 ;
		Val = 0 ;
	} ;
	
	
	MVT_dat(const char *name, double *val, enum Component_id comp){
		Name = strdup(name) ;
		Val = val ;
		Comp = comp ;
	} ;
	
	
	MVT_dat(MVT_dat &src) {
		
		NodeUsrDat::operator=(src) ;

		if(src.Name == 0)
			Name = 0 ;
		else
			Name = strdup(src.Name) ;
	
		Val = src.Val ;
		Comp = src.Comp ;
	} ;
	
		
	void operator=(MVT_dat &src) {
		
		if(this != &src){
			NodeUsrDat::operator=(src) ;

			if(Name != 0) 
				free(Name) ;
			Name = 0 ;
			if(src.Name != 0)
				Name = strdup(src.Name) ;
			Val = src.Val ;
			Comp = src.Comp ;
		}
	} ;
	
	
	void Print(ostream &out){
	
		out << Name << " " << *Val ;
	} ;
	
	
	char *GetKey(void) {
	
		return(Name) ;
	} ;
		
		/* compare > 0 => str > this */ 
	int Compare(const char *str){
	
		return (strcmp(Name, str)) ;
	} ;

	enum Component_id Comp ;
	double *Val ;
	char *Name ;	
} ;






double *VarTbl::Search(const char *Name){
	  /* pre: true
	    post: (Search(Name) == 0) => Name not present
			  ELSE Search(Name) == pointer to value of Name */
	
  MVT_dat *dat ;
	dat = (MVT_dat *)BSTree::Search(Name) ;
	if(dat != 0) 
		return (dat->Val) ;
  
	return(0) ; 
} ;
	
	
Component_id VarTbl::GetId(const char *Name){
	  /* pre: (Search(Name) != 0)
	    post: GetId(Name) == component id of name */
	
  MVT_dat *dat ;
	dat = (MVT_dat *)BSTree::Search(Name) ;
	if(dat != 0) 
		return (dat->Comp) ;
  
} ;

void VarTbl::Add(const char *Name, double *val, Component_id comp) {
	  /* pre: Search(Name) == 0 ;
	    post: Search(Name) == val */
		
  MVT_dat *dat = new MVT_dat(Name, val, comp) ;
  BSTree::Add((NodeUsrDat *) dat) ;
} ;




