

/*****************************************************************************

ScrumPy -- Metabolic Modelling with Python

Copyright Mark Poolman 1995 - 2002

 This file is part of ScrumPy.

    ScrumPy is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    ScrumPy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ScrumPy; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


*****************************************************************************/

#ifndef SCAMPI_MOD_B
#define SCAMPI_MOD_B



#ifdef __cplusplus
#define extern extern "C"
#endif


typedef struct SS_ScratchPad *SS_ScratchPad_t ;
typedef struct Integ_ScratchPad *Integ_ScratchPad_t ;


typedef struct ScampiMod {

	const char *Name ;
	
	int n_Reacts ;
	const char **React_Names ;
	double (**Rate_Eqns)(void),
		*React_Rates ;
	
	int n_Mets, n_Cons, n_Free ;
	
	const char **Met_Names ;
	double (**Met_ODEs)(void) ;
	double	(**Met_Cons_Eqns)(void) ;
	double	*dMet_dT,
		*Met_Vals ;
		
	int n_Params ;
	const char **Param_Names ;
	double *Param_Vals ;

	double *Time ;
		
	SS_ScratchPad_t SS_ScratchPad ;
	Integ_ScratchPad_t Integ_ScratchPad ;

} *ScampiModel_t, *ScampiModel_ptr,  ScampiModel ;
	
		

enum Component_id {Time, Param, Conc, Vel, dMet} ;


#ifdef WANT_SPI_ERR_MSG

char *SPI_errMsgs[] = {
        "No Error",
        "Negative concentration",
        "Requested tolerance not achieved",
        "Solver can't get steady state with these start conditions",
        "Simulation can't proceed from this position",
        "Out of memory",
        "Couldn't open file",
        "No more data",
        "Model not defined" } ;

#else

extern char *SPI_errMsgs[] ;

#endif










enum SPI_err{ OK, NegConc, OutOfTol, SingMat, IntegError, 
		NoMem, BadFile, NoDat, Undefined } ;


extern int ModGetVecSize(ScampiModel_t md, enum Component_id comp) ;
	/* pre : md valid, 
	  post : GetMDvecSize(md, comp) = number of elememts in md of type comp **/

extern double *ModGetVec(ScampiModel_t md, enum Component_id comp) ;
    /* pre: comp != Time
      post: internal vector corresponding to comp */

extern void  ModSetVec(ScampiModel_t model, enum Component_id comp, const double *vec) ;
	/* pre : md valid, vec is length GetMDvecSize(md, comp)
	  post : *GetMDvec(md, comp) == vec */

extern void ModEval(ScampiModel_t model) ;
	/* pre: TRUE
	  post: velocities and dependent metabolites updated to 
	  	    reflect current params and concs */ 
		
extern int /* bool */ HasNegConc(ScampiModel_t mod) ;
	/* pre: TRUE
	  post: HasNegConc(mod) => mod has one or more negative concentrations */




#ifdef __cplusplus
#undef extern
#endif

#endif
