

/*****************************************************************************

ScrumPy -- Metabolic Modelling with Python

Copyright Mark Poolman 1995 - 2002

 This file is part of ScrumPy.

    ScrumPy is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    ScrumPy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ScrumPy; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


*****************************************************************************/
/* 
M.Poolman  03/01/95   Model description programmers interface for SCAMP
internal clone version scamp call_simul function, tested under ultrix 
and Amiga
*/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h> 
#include <unistd.h>
#include <dlfcn.h>



#include "ScampiModelBase.h"
#include "SS_ScratchPad.h"
#include "Integ_ScratchPad.h"
#include "Scampi.h" 


	/***** private general purpose functions *****/


enum SPI_err NewScratchPads(ScampiModel_t Mod){
  
  
	Mod->SS_ScratchPad = (SS_ScratchPad_t) 
		NewSS_ScratchPad(Mod) ;
		
	Mod->Integ_ScratchPad = (Integ_ScratchPad_t) 
		NewInteg_ScratchPad(Mod) ;

	if(Mod->SS_ScratchPad && Mod->Integ_ScratchPad)
		return OK ;
	else
		return NoMem ;
				
}
		




ScampiModel_t NewMD(const char *modname, const char *libname, void **dlib_p) {
	/* pre : modname is name of model contained in the .so file libname 
	  post : !NULL return is valid instance of ScampiModel_t,
	         dlib_p points dynamic library holding the model */

 
  ScampiModel_t *rvp  ;
  void *dlib ;
  
	dlib = dlopen (libname, RTLD_NOW) ;
	if(!dlib){
		fprintf(stderr,"\n!!\n!! Load_so - dlopen(%s) failed because %s !!\n!!\n", libname, dlerror()) ;
		return(0) ;
	}
	
	rvp = (ScampiModel_t *) dlsym(dlib, modname) ;
	if(!rvp){
		fprintf(stderr, "\n!!\n!! Load_so - dlsym(%s) failed because %s !!\n!!\n\n", modname, dlerror()) ;
		return(0) ;
	}
	else {
		*dlib_p = dlib ;
		NewScratchPads(*rvp) ;
		return *rvp ;
	}
} ;



void OneShotSim(ScampiModel_t Model, double time, double tol,
                       enum SPI_err *err){
	/* pre: tol > 0.0 ; vestigial tol, ignored
      post: (err == OK) => model simulated between model  current time and time,
                        ELSE error as defined by err, model unchanged */

    ModEval(Model) ;
	Integrate(Model->Integ_ScratchPad, time, err) ;     /* integrate */
    if(*err == OK){
        if(HasNegConc(Model))
           *err = NegConc ;
    }
    if(*err != OK)             /* reset the simulator if a problem */
        ResetIntegrator(Model) ;
	else 
		*(Model->Time) = time ; 
} ;

	
	

extern void ResetIntegrator(ScampiModel_t Model) {
	ResetInteg_ScratchPad(Model->Integ_ScratchPad) ;
}    /* just exporting the otherwise private function - 
        Scampi should be the *only* only point of contact between  
		the lower level stuff and the outside world
		-- might also want to do other stuff at this point */ 

		
		
		
	

double Find_ss(ScampiModel_t model, int MaxIters, double tol, enum SPI_err *err) {
	/* pre : scampi.InitModelDesc(), MaxIters > 1
	  post : (err == OK) 		=> model is in steady state to whithin tolerance tol
	  								and all concentrations are +ve, returns tollerance achieved
	         (err == NegConc)	=> model is in steady state to whithin tolerance tol
	         						but contains >= 1 -ve concentrations
	         (err == OutOfTol)	=> failed to get whithin tol after MaxIters iterations
	          						state of model not known
			 (err = SingMat)	=> singular Jacobian, insoluble model 			 
	  */
	  double rv ;
	  rv = Solve(model->SS_ScratchPad, MaxIters, tol, err) ;
	  fprintf(stderr, "Find_ss %d %g\n", *err,rv) ;
	  /*fprintf(stderr, "Find_ss %g\n", rv) ;*/
  return (rv) ;
}

			

