

/*****************************************************************************

ScrumPy -- Metabolic Modelling with Python

Copyright Mark Poolman 1995 - 2002

 This file is part of ScrumPy.

    ScrumPy is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    ScrumPy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ScrumPy; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


*****************************************************************************/

#ifndef SCAMPI_H
#define SCAMPI_H

#include "ScampiModelBase.h"



#ifdef __cplusplus
#define extern extern "C"
#endif




extern ScampiModel_t NewMD(const char *modname, const char *libname, void **dlib_p) ;
	/* pre : modname is name of model contained in the .so file libname 
	  post : !NULL return is valid instance of ScampiModel_t */



extern double Find_ss(ScampiModel_t model, int MaxIters, double tol, enum SPI_err *err) ;
	/* pre : scampi.InitModelDesc(), MaxIters > 1
	  post : (err == OK) 		=> model is in steady state to whithin tolerance tol
	  								and all concentrations are +ve
	         (err == NegConc)	=> model is in steady state to whithin tolerance tol
	         						but contains >= 1 -ve concentrations
	         (err == OutOfTol)	=> failed to get whithin tol after MaxIters iterations
	          						state of model not known
			 (err = SingMat)	=> singular Jacobian, insoluble model 
			 returns tollerance achieved
	  */


		// +++++++++++++++++++++++++++++++++++++++++++
extern void OneShotSim(ScampiModel_t Model, double time, double tol, 
						enum SPI_err *err) ;
	/* pre: tol > 0.0 ;
	  post: (err == OK) => model simulated between model  current time and time,
	  		ELSE error as defined by err, model unchanged */
			
extern void ResetIntegrator(ScampiModel_t Model) ;					


#ifdef __cplusplus
#undef extern
#endif


#endif
