

/*****************************************************************************

ScrumPy -- Metabolic Modelling with Python

Copyright Mark Poolman 1995 - 2002

 This file is part of ScrumPy.

    ScrumPy is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    ScrumPy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ScrumPy; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


*****************************************************************************/
#include<stdio.h>
#include<stdlib.h>

#include "ScampiModelBase.h"


#ifndef TRUE
#define FALSE 0
#define TRUE !FALSE
#endif





int ModGetVecSize(ScampiModel_t md, enum Component_id comp) {
    /* pre : md valid,
      post : GetMDvecSize(md, comp) = number of elememts in md of type comp **/

  int rv ;

    switch(comp){
        case Param:
            rv = md->n_Params ;
            break ;
        case Conc:
            rv = md->n_Mets ;
            break ;
        case Vel:
            rv = md->n_Reacts ;
            break ;
        default:
            fprintf(stderr, "!! bad comp = %d in GetMDvecSize !!\n", comp) ;
            break ;
    }
    return(rv) ;
}




double *ModGetVec(ScampiModel_t md, enum Component_id comp){

  double *rv ;

    switch (comp){
        case Param:
            rv = md->Param_Vals ;
            break ;
        case Conc:
            rv = md->Met_Vals ;
            break ;
        case Vel:
            rv = md->React_Rates ;
            break ;
        case dMet:
            rv = md->dMet_dT ;
            break ;
        default:
            fprintf(stderr, "!!\n!! Bad Comp = %d in GetVec() !!\n!!\n", comp) ;
            break ;
    }
    return(rv) ;
}




void  ModSetVec(ScampiModel_t model, enum Component_id comp, const double *vec) {
    /* pre : md valid, vec is length GetMDvecSize(md, comp)
      post : *GetMDvec(md, comp) == vec */

  double  *orig ;
  int n, n_elems ;

    n_elems = ModGetVecSize(model, comp) ;  /* how long is the array */
    orig = ModGetVec(model, comp) ;         /* where is the original */
    for(n = 0 ; n < n_elems ; n++) orig[n] = vec[n] ; /* copy vec to orig */
} ;




int HasNegConc(ScampiModel_t mod) {

  int n ;

     for(n = 0 ; n < mod->n_Mets ; n++)
        if(mod->Met_Vals[n] < 0.0) return(TRUE) ;

     return(FALSE) ;
}



void ModEval(ScampiModel_t mod){

  int n ;

    for(n = 0 ; n < mod->n_Cons ; n++){
        mod->Met_Vals[n+mod->n_Free] = mod->Met_Cons_Eqns[n]() ;
    }

    for(n =  0 ; n < mod->n_Reacts ; n++){
        mod->React_Rates[n] = mod->Rate_Eqns[n]() ;
    }

    for(n = 0 ; n < mod->n_Free ; n++){
        mod->dMet_dT[n] = mod->Met_ODEs[n]() ;
        /* fprintf (stderr, "ode %s = %g\n", mod->Met_Names[n], mod->dMet_dT[n]); */
    }
}




void ModEvalNC(ScampiModel_t mod){

  int n ;

    for(n =  0 ; n < mod->n_Reacts ; n++){
        mod->React_Rates[n] = mod->Rate_Eqns[n]() ;
    }

    for(n = 0 ; n < mod->n_Free ; n++){
        mod->dMet_dT[n] = mod->Met_ODEs[n]() ;
        /* fprintf (stderr, "ode %s = %g\n", mod->Met_Names[n], mod->dMet_dT[n]); */
    }
}



