import sys
import ScrumPy
from ScrumPy.Util import Format


TOL=1e-6



def MakeSM(smx):
    
    sm = smx.Copy()
    for met in smx.Externs:
	sm.DelRow(met)
    sm.Externs = smx.Externs[:]
    
    return sm


def AreOrthogonal(a,b,tol=TOL ):

    theta = abs(ScrumPy.Util.Seq.CosTheta(a,b))

    return abs(theta) <tol


def FindDeads(m, txs=[], tol=TOL):
    """ identify dead reactions as those that are orthogonal to all transporters in txs.
        See Poolman et al 2008 JTB """

    
    k = m.sm.OrthNullSpace()
    lentxs = len(txs)
    rv = []

    for reac in k.rnames:
	if not reac in txs:
	    dead = max(map(abs, k[reac])) < tol
	    nchecked = 0
	    
	    while not dead and nchecked < lentxs:
		tx = txs[nchecked]
		nchecked +=1
		dead = AreOrthogonal(k[tx],k[reac],tol)
		
	    if dead:
		rv.append(reac)

    return rv
	

def DelDeads(m,txs=[],tol = TOL):

    deads = FindDeads(m, txs, tol)
    m.DelReactions(deads)

    return deads



def HasSubsets(ess):

    for ss in ess:
        if len(ess[ss]) >1:
            return True
    return False

def RenameEss(ess,sfx):

    for k in ess.keys():
        if len(ess[k])>1:
            ess[k+"_"+str(sfx)] = ess[k]
            del ess[k]
        


def _Condense(m, IsosEss):
    
    isos = m.DelIsoforms()
    e = m.EnzSubsets()
    del e["DeadReacs"]
    nits = len(IsosEss)
    RenameEss(e, nits)

    IsosEss.append([isos,e])
    print nits

    
    emtx = e.ToMtx()
    #emtx.DelCol("DeadReacs")
    smx = m.smexterns.Mul(emtx)
    smx.Externs = m.sm.Externs[:]
    smx.ZapZeroes()
    smx.Irrevs = e.Irrev_ss()

    m.smexterns=smx
    m.sm = MakeSM(smx)

    CanCondense = len(isos) >0 or HasSubsets(e)
    CanCondense = CanCondense and len(m.sm) >0  # check len(m.sm) just in case we've eliminated ALL internal mets
    

    if CanCondense:  
	_Condense(m,IsosEss)

 

def Condense(m,txs=[],tol=TOL):

    deads =  DelDeads(m, txs,tol)
    print len(deads), "dead"

    m.sm.ReQuote()
    m.smexterns.ReQuote()
    IsosEss = []
    _Condense(m, IsosEss)
    IsosEss[0][1]["DeadReacs"] = deads

    
    m.smexterns.ToScrumPy(m.ModelName+"_Condensed.spy")

    open(m.ModelName+"_Condensed.py","w").write(Format.FormatNested(IsosEss,Quoted=False))

    m.sm.DeQuote()
    m.smexterns.DeQuote()                                
                                                

    

    return IsosEss

    
                              

    
        
        
            
        
    
