

"""

ScrumPy -- Metabolic Modelling with Python

Copyright Mark Poolman 2006 -

 This file is part of ScrumPy.

    ScrumPy is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    ScrumPy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ScrumPy; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""

"""
Common entry point for "scientific" functions, especially on vectors and matrices.
Currently using numpy, but this may change in future.
"""


import math,sys

import numpy

from ScrumPy.Util import Seq




matrix = numpy.matrix
vector = numpy.array
svd = numpy.linalg.svd
pinv = numpy.linalg.pinv

#tx =  Numeric.transpose
#diag = Numeric.diagonal
#array = Numeric.array
#svd = LinearAlgebra.singular_value_decomposition


TOL=1e-12 # smallest difference between two numbers that can be considered non-zero

def PolyFit(Xdat, Ydat, deg=1):
    "polynomial fit of X,Y of degree deg"

    return numpy.polyfit(Xdat,Ydat,deg)



def null(A,tol=TOL,AsList=False):
    "orthonormal nullspace, elements are floats"

    u,w,vt=svd(A,full_matrices=1)
    v,i = Seq.PosOfFirst(w.tolist(),lambda x: x <= tol)
    if i == -1:
        i = len(w)
    rv = vt[i:,].transpose()

    if AsList:
        return rv.tolist()
    return rv

def mul(a,b,AsList=False):

    return a*b


def Mod(vec):
    return math.sqrt(numpy.vdot(vec,vec))


def CosTheta(veca,vecb):
    rv = numpy.vdot(veca,vecb)/(Mod(veca)*Mod(vecb))
    if rv >1.0:
        rv = 1.0
    elif rv < -1.0:
        rv = -1.0

    return rv


AbsCosTheta = lambda a,b: abs(CosTheta(a,b))

def AreParallel(a,b, tol=TOL):
    return abs(1-CosTheta(a,b)) <= tol

def AreOrthogonal(a,b,tol = TOL):
    return abs(CosTheta(a,b)) <= tol


Theta    = lambda a,b : math.acos(CosTheta(a,b))
AbsTheta = lambda a,b : abs(Theta(a,b))
Q1Theta  = lambda a,b : math.acos(AbsCosTheta(a,b))






