"""


ScrumPy -- Metabolic Modelling with Python

Copyright Mark Poolman 1995 - 2002

 This file is part of ScrumPy.

    ScrumPy is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    ScrumPy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ScrumPy; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
from ScrumPy.ThirdParty.idle import PyShell, EditorWindow
import Tkinter

from Pmw import MegaToplevel

import SimDatReq
import Messages
import Inspector
import Plotter
import Saver
import FileDialogs


#$ event <<compile>>
#$ win <Alt-c>
#$ unix <Alt-c>


def NewWin(root):
    try:
        rv = Pmw.MegaToplevel(root)
    except:
        rv = Tkinter.Toplevel
    rv.withdraw()
    return rv


class EdWin(EditorWindow.EditorWindow):
        def __init__(self, Model=None, Root=None, *args, **argsl):

            EditorWindow.EditorWindow.__init__(self,
                                                 PyShell.flist,
                                                 Model.FileName,
                                                 None,
                                                 PyShell.root)
            self.model = Model
            self.guiroot = Root
            self.MakeMenu()
            self.text.bind("<<compile>>",self.MenuCompile)


        def CheckUsable(self):
            if self.model.Usable():
                return 1

            msg = "Model " + self.model.ModelName +" is not usable\n"
            msg += "try Menu/ScrumPy/Compile first"
            Messages.ErrorMsg(Parent=self.guiroot, Message=msg)
            return 0


        def SaveFile(self):
            self.menubar.children["file"].invoke(8)

        def MenuCompile(self):

            self.model.Reload()

        def MenuSaveSBML(self):
            fname = FileDialogs.SaveFileName()
            self.model.SaveSBML(str(fname)) # str because libSBML barfs on unicode


        def MenuSimulate(self):

            if self.CheckUsable():
                SimDatReq.SimRequest(
                    None, #self,#NewWin(self.guiroot),
                    self.model,
                    self.model.NSteps,
                    self.model.GetTincr())


        def MenuSolve(self):
            if self.CheckUsable():
                self.model.FindSS()
                if self.model.IsOK():
                    Messages.InfoMsg(Message="Got steady-state")
                else:
                    Messages.ErrorMsg(Message="Failed to get steady-state\n"+
                                      self.model.ErrorMsg())

        def MenuInspector(self):
            if self.CheckUsable():
                self.model.AddStatMonitor(
                    Inspector.Inspector(self.model,self.model.ModelName))

        def MenuDynMonitor(self):
            if self.CheckUsable():
                self.model.AddDynMonitor(
                    Plotter.Plotter(Model=self.model,
                                    Root=self.guiroot,
                                    xname="Time",
                                    Title=self.model.ModelName))

        def MenuSaver(self):
            if self.CheckUsable():
                Saver.Saver(self.model)




        def MakeMenu(self):

            Menu = Tkinter.Menu(PyShell.root)
            Menu.add("command",{"label":"Compile", "command":self.MenuCompile})
            Menu.add("command",{"label":"Save SBML", "command": self.MenuSaveSBML})
            #Menu.add("command",{"label":"Simulate", "command":self.MenuSimulate})
            #Menu.add("command",{"label":"Solve", "command":self.MenuSolve})
            Menu.add("command",{"label":"Inspector", "command":self.MenuInspector})
            #Menu.add("command",{"label":"Dynamic Monitor", "command":self.MenuDynMonitor})
            #Menu.add("command",{"label":"Saver", "command":self.MenuSaver})
            self.menubar.add("cascade",{"label":"ScrumPy","menu":Menu})


