"""


ScrumPy -- Metabolic Modelling with Python

Copyright Mark Poolman 1995 - 2002

 This file is part of ScrumPy.

    ScrumPy is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    ScrumPy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ScrumPy; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
import Tkinter

#from ScrumPy.ThirdParty.Pmw
import Pmw

class Inspector(Tkinter.Toplevel):
    
    def __init__(self,Model, Label=""):

        Tkinter.Toplevel.__init__(self)
        self.model = Model

        l = Tkinter.Label(self, text="Inspecting " + Label)
        l.pack()

        self.buttonBox = Pmw.ButtonBox(             # box for user actions
            self,
            labelpos = 'nw',
            label_text = 'Actions:',
            frame_borderwidth = 2,
            frame_relief = 'groove'
        )

        self.buttonBox.pack(fill = 'x',
                            padx = 10,
                            pady = 10)
        self.buttonBox.add('Set', command = self.Set)
        self.buttonBox.add('Get', command = self.Get)
        self.buttonBox.add("Reset", command = self.Reset)


        self.optionbox = Pmw.RadioSelect(self, # options exercised user clicks "set" 
		buttontype = 'checkbutton',
		orient = 'horizontal',
		labelpos = 'nw',
		command = self.OptionToggle,
		label_text = 'Options:',
		hull_borderwidth = 2,
		hull_relief = 'groove'
	)
	self.optionbox.pack(fill = 'x',padx = 10, pady = 10)	
        for label in ('Recalculate Cons', 'Evaluate', 'Solve'):
	    self.optionbox.add(label)
	self.Recalc = 0
	self.Eval = 0
	self.Solve = 0

        self.frame = Pmw.ScrolledFrame(self,
		labelpos = 'n', #label_text = 'Inspecting '+Label,
		usehullsize = 1,
		hull_width = 400,
		hull_height = 220,
	)
        
        self.counters = []
        self.Reset()



    def Reset(self):
        
        for c in self.counters:
            c.destroy()

        self.counters = []

        for k in self.model.keys():
            counter = Pmw.Counter(
                self.frame.interior(),
                labelpos="w",
                label_text=k,
                entryfield_value = self.model[k],
                entryfield_validate = {
                    'validator' : 'real',
                    'min' : 0, 'minstrict' : 0},
                increment = self.model[k]/20.0,
                datatype = {'counter' : 'real'}
            )
            counter.pack(expand=1,fill="both")
            self.counters.append(counter)
        Pmw.alignlabels(self.counters)
        self.frame.pack(expand=1, fill="both")

        

    def Set(self):
        keys = self.model.keys()
        for k in range (len(keys)):
            self.model[keys[k]] = float(self.counters[k].get())

        if self.Recalc:
            self.model.InitCSums()
        if self.Eval:
            self.model.Eval()
        if self.Solve:
            self.model.FindSS()
        self.model.UpdateDynMons()
        self.model.UpdateStatMons()
        
        

    def Get(self):
        keys = self.model.keys()
        for k in range (len(keys)):
            self.counters[k].setentry(self.model[keys[k]])

    def Update(self):
        self.Get()


    def OptionToggle(self, tag, state):
        t = tag[0]
        if t == "R":
           self.Recalc = not self.Recalc    # just toggle boolean flags
        elif t == "E":
            self.Eval = not self.Eval
        elif t == "S":
            self.Solve = not self.Solve
        else:
            raise "UnknownOption", "in "+__file__+ " optionbox_cb "+ tag 
     

    def destroy(self):
        try:
            self.model.Unregister(self)
        except:
            pass

        Tkinter.Toplevel.destroy(self)
            
                

            
            
                
