"""


ScrumPy -- Metabolic Modelling with Python

Copyright Mark Poolman 1995 - 2002

 This file is part of ScrumPy.

    ScrumPy is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    ScrumPy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ScrumPy; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""

import Tkinter



import Messages
import FileDialogs

import TextSelect

#from ScrumPy.ThirdParty.Pmw.Pmw import MegaToplevel
#from ScrumPy.ThirdParty.Pmw
import Pmw
#from ScrumPy.Data import DataSets


def AntiBugKludge():
    # for reasons as yet (30/09/02) unknown, the text selector fails to instatiate
    # when the plotter's root is gui.root (which should be the ultimate root for everything)
    # hence we simply make a new Tkinter.Toplevel to act as root and withdraw it

    rv = Tkinter.Toplevel()
    rv.withdraw()
    return rv



class Plotter(Pmw.MegaToplevel):
    def __init__(self, Root=None,  Model=None, xname="", Title="",xrange=1, xinc=0.1):

        self.xdat = None
        self.ydat = None        # yname inited by NewPlot so that we can
        self.yname = None       # change them later w/o creating a new instance
        self.xrange = xrange    # likwise these fields are brought into existence here
        self.xinc = xinc        # but are really inited by SetXScales

        self.model = Model      # Model can be any dictionary-like instance
        if xname=="":
            xname = Model.keys()[0]  # arbitrary default if not supplied

        self.xname=xname

        Root = AntiBugKludge()
        Pmw.MegaToplevel.__init__(self,Root)


        self.graphgroup = Pmw.Group(self._hull)

        self.graph = Pmw.Blt.Stripchart(self.graphgroup.interior(),title=Title)    # where we plot the data
        self.graphscale = Tkinter.Scale(self.graphgroup.interior(),orient="horizontal")
        self.yselect = TextSelect.TextSelector(                     # where user decides what to plot
            Root=self._hull,
            entries=self.model.keys(),
            dblclickfcn=self.NewYSelect,
            label_text = "Double-click to select",
            labelpos = "n")


        self.buttonBox = Pmw.ButtonBox(                        # box for user options
            self._hull,
            labelpos = 'nw',
            label_text = 'Options:',
            frame_borderwidth = 2,
            frame_relief = 'groove')
        self.buttonBox.pack(fill = 'both', padx = 10, pady = 10)
        self.buttonBox.add('Save as Text', command = self.SaveText)
        self.buttonBox.add('Save as Postscript', command = self.SavePS)
        self.buttonBox.add("Reset", command = self.Reset)


        self.graph.pack(side="top",expand=1, fill="both")
        self.graphscale.pack(side="bottom", expand=1, fill="both")
        self.graphgroup.pack(side="left", expand=1, fill="both")
        self.yselect.pack(side = "right",expand=1, fill="both")

        self.NewPlot(xname, xrange, xinc, self.yname)

    def destroy(self, FromModel=0):
        try:
            self.model.Unregister(self)
        except:
            pass

        Pmw.MegaToplevel.destroy(self)




    def line_delete(self, name, **kw):
        self.graph.tk.call((self.graph._w, 'line', 'delete', name) + self.graph._options(kw))

    def NewPlot(self, xname, xrange, xinc, yname):


        try:
            del self.xdat
        except:
            pass  # don't try to delete twice
        try:
            del self.ydat
        except:
            pass

        if self.yname != None:
            self.line_delete(self.yname)

        self.xname = xname
        self.xdat = Pmw.Blt.Vector(master=self._hull)
        if yname != None:
            self.yname = yname
        elif self.yname == None:                # if we haven't been given and don't already have
            self.yname = self.model.keys()[0]   # a yname then use an arbitrary, but valid, default

        self.ydat = Pmw.Blt.Vector(master=self._hull)

        self.SetXScales(xinc, xrange)

        self.graph.line_create(self.yname,
                               xdata=self.xdat,
                               ydata=self.ydat,
                               dashes=0,
                               linewidth=1,
                               symbol='')

        self.Update()

    def SetXScales(self,xinc, xrange):
        self.xinc = xinc
        self.xrange = xrange
        self.graph.axis_configure("x",shiftby=xinc,autorange=xrange)
        self.graphscale["resolution"] = xinc
        self.graphscale["to"] = xrange
        self.graphscale.pack(side="bottom", expand=1, fill="both")


    def NewSim(self, xinc, nsteps):
        # called by clients only, needs to set x behaviour to look pretty
        # not yet sure of best way to do it

        self.SetXScales(xinc, self.xrange)






    def Update(self):
        xdat = self.model[self.xname]
        self.xdat.append(xdat)
        self.ydat.append(self.model[self.yname])
        self.update()
        self.update_idletasks()


    def NewYSelect(self, ynames):
        if len (ynames) != 1:
            raise "ScrumPy.GUI.Plotter.NewYSelect", "Got " + str(len(ynames)) + " ynames, should have been 1"

        self.NewPlot(self.xname, self.xrange, self.xinc, ynames[0])

    def SaveText(self):
        #ds = DataSets.DataSet()
        #ds.NewCol(self.xdat,self.xname)
        #ds.NewCol(self.ydat,self.yname)
        #ds.WriteFile(FileDialogs.SaveFileName())
        print "SaveText not implemented" #  incompatible withnew wx GUI




    def SavePS(self):
        ps = self.graph.postscript_output()
        try:
            f = FileDialogs.SaveFile()
            f.write(ps)
        except:
            Messages.ErrorMsg(Message = "Couldn't write postscript file\nCheck file-name/path")


    def Reset(self):
        #self.yselect.destroy()                    # key names might have chnaged, so destroy and
        names = self.model.keys()                 # regenerate a new selctor
        self.yselect.setlist(names)


        if  self.xname in names:
            xname = self.xname
        else:
            xname = names[0]     # old xname has vannished, arbitrary but valid default

        if self.yname in names:
            yname = self.yname
        else:
            yname = names[1]

        self.NewPlot(xname, 1, 0.1, yname)


m = {}
for c in range(ord("a"),ord("z")): # a handy and convenient model for testing purposes
    m[chr(c)] = c









