"""


ScrumPy -- Metabolic Modelling with Python

Copyright Mark Poolman 1995 - 2002

 This file is part of ScrumPy.

    ScrumPy is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    ScrumPy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ScrumPy; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
import Tkinter

#from ScrumPy.ThirdParty.Pmw
import Pmw

import TextSelect
import FileDialogs

def ignore(*args, **kwargs):
    pass

class Saver(Tkinter.Toplevel):
    def __init__(self, Model, **kwargs):

        self.model = Model
        try:
            self.name = Model.ModelName
        except:
            self.name = "un-named data"
        self.TheData = {}  # make a dict here in case user tries to update before self.OK()
        Tkinter.Toplevel.__init__(self)
        self.group = Pmw.Group(self)

        self.typesel = Pmw.RadioSelect(
            self.group.interior(),
            command=self.RadioClick,
            #label_text = 'Update policy',
            buttontype="radiobutton",
            orient="vertical"
        )
        self.typesel.add("Static")
        self.typesel.add("Dynamic")

        self.datsel = TextSelect.TextSelector(self.group.interior(), self.model.keys(), ignore)
        self.datsel._listbox["selectmode"] = "extended"

        self.ok = Tkinter.Button(self, text="OK", command=self.OK)
        self.group.pack(expand=1, fill="both")

        self.typesel.pack(side = "left", expand=1, fill="both")
        self.datsel.pack(side = "right", expand=1, fill="both")
        self.ok.pack(side="bottom", fill="both")


    def RadioClick(self,arg):

        if arg == "Static":
            self.MakeStatic()
        else:
            self.MakeDynamic()

    def MakeStatic(self):
        self.model.Unregister(self)
        self.model.AddStatMonitor(self)

    def MakeDynamic(self):
        self.model.Unregister(self)
        self.model.AddDynMonitor(self)

    def OK(self):
        self.datanames = self.datsel.Selected()                # what the user has selected
        self.TheData = self.model.NewDataSet(self.datanames)   # where the data will be stored
        self.fname = FileDialogs.SaveFileName()                # file name to which data will finally go
        self.typesel.destroy()                                 # user has selected, can't change mind
        self.ok.destroy()                                      # you can only say "OK" once
        self.datsel.setlist(self.datanames)                     # so that the user can see what they selected
        self.Running = Tkinter.Label(self, text="Saver for " + self.name + "\n(will update automatically)" )
        self.Running.pack()  # leave a window open to remind them they have an active saver


    def destroy(self):
        self.model.Unregister(self)
        self.SaveOrLoose()

    def SaveOrLoose(self):
        try:
            self.Running.destroy()
            msg = Tkinter.Label(
                self,
                text="Saver for " + self.name +
                "\nModel has changed or window closed:\n save or loose current data ?")
            msg.pack()
            Tkinter.Button(self, text="Save", command=self.finalSave).pack()
            Tkinter.Button(self, text="Loose", command=self.tkdestroy).pack()
        except:
            self.tkdestroy() # if the user destroys before hitting "OK"

    def Save(self):
        self.TheData.WriteFile(self.fname)


    def tkdestroy(self):
        Tkinter.Toplevel.destroy(self)



    def finalSave(self):
        self.Save()
        self.tkdestroy()


    def Update(self):
        self.TheData.Update()
        #self.model.UpdateDataSet(self.TheData)


