
import types

import wx
from wx.lib  import plot

cdb = wx.TheColourDatabase
PenColours=[ # http://docs.wxwidgets.org/stable/wx_wxcolourdatabase.html#wxcolourdatabase
'AQUAMARINE', 'BLACK', 'BLUE', 'BLUE VIOLET', 'BROWN', 'CADET BLUE', 'CORAL', 'CORNFLOWER BLUE', 'CYAN', 'DARK GREY', 'DARK GREEN', 'DARK OLIVE GREEN', 'DARK ORCHID', 'DARK SLATE BLUE', 'DARK SLATE GREY DARK TURQUOISE', 'DIM GREY', 'FIREBRICK', 'FOREST GREEN', 'GOLD', 'GOLDENROD', 'GREY', 'GREEN', 'GREEN YELLOW', 'INDIAN RED', 'KHAKI', 'LIGHT BLUE', 'LIGHT GREY', 'LIGHT STEEL BLUE', 'LIME GREEN', 'MAGENTA', 'MAROON', 'MEDIUM AQUAMARINE', 'MEDIUM BLUE', 'MEDIUM FOREST GREEN', 'MEDIUM GOLDENROD', 'MEDIUM ORCHID', 'MEDIUM SEA GREEN', 'MEDIUM SLATE BLUE', 'MEDIUM SPRING GREEN', 'MEDIUM TURQUOISE', 'MEDIUM VIOLET RED', 'MIDNIGHT BLUE', 'NAVY', 'ORANGE', 'ORANGE RED', 'ORCHID', 'PALE GREEN', 'PINK', 'PLUM', 'PURPLE', 'RED', 'SALMON', 'SEA GREEN', 'SIENNA', 'SKY BLUE', 'SLATE BLUE', 'SPRING GREEN', 'STEEL BLUE', 'TAN', 'THISTLE', 'TURQUOISE', 'VIOLET', 'VIOLET RED', 'WHEAT', 'WHITE', 'YELLOW', 'YELLOW GREEN']

PenStyleKeys= ['SOLID', 'TRANSPARENT', 'DOT', 'LONG_DASH', 'SHORT_DASH', 'DOT_DASH', 'STIPPLE', 'USER_DASH', 'BDIAGONAL_HATCH', 'CROSSDIAG_HATCH', 'FDIAGONAL_HATCH', 'CROSS_HATCH', 'HORIZONTAL_HATCH', 'VERTICAL_HATCH']
PenStyleCodes = map(lambda k:  getattr(wx, k),  PenStyleKeys)
PenStyleDic   = dict(zip(PenStyleKeys,  PenStyleCodes))

MarkerStyleKeys =[
  'TRANSPARENT',  # : Transparent (no fill).
  'SOLID', # Solid.
  'STIPPLE', # Uses a bitmap as a stipple.
  'BDIAGONAL_HATCH', # Backward diagonal hatch.
  'CROSSDIAG_HATCH', # Cross-diagonal hatch.
  'FDIAGONAL_HATCH', # Forward diagonal hatch.
  'CROSS_HATCH', # Cross hatch.
  'HORIZONTAL_HATCH', # Horizontal hatch.
  'VERTICAL_HATCH' # Vertical hatch.
]
MarkerStyleCodes=  map(lambda k: getattr(wx, k),  MarkerStyleKeys)
MarkerStyleDic = dict(zip(MarkerStyleKeys,  MarkerStyleCodes))


MarkerShapes = ['circle',  'dot', 'square',  'triangle', 'triangle_down', 'cross', 'plus']

class Plotter(plot.TestFrame):
    def __init__(self, parent=None, id=-1, title="ScrumPy"):
        wx.Frame.__init__(self, parent, id, title,
                          wx.DefaultPosition, (600, 400))

        # Now Create the menu bar and items
        self.CreateMenu()

        # A status bar to tell people what's happening
        self.CreateStatusBar(1)

        self.client = plot.PlotCanvas(self)
        #define the function for drawing pointLabels
        self.client.SetPointLabelFunc(self.DrawPointLabel)
        # Create mouse event for showing cursor coords in status bar
        self.client.canvas.Bind(wx.EVT_LEFT_DOWN, self.OnMouseLeftDown)
        # Show closest point when enabled
        self.client.canvas.Bind(wx.EVT_MOTION, self.OnMotion)

        self.PlotItems={}

        self.Show(True)


    def HasItem(self, id):
        return self.PlotItems.has_key(id)

    def GetItemIds(self):
        return self.PlotItems.keys()

    def DelItem(self,  id):
        del self.PlotItems[id]


    def AddLinePlotItem(self, x, y, Name,  colour="BLACK",  style="SOLID",  width=2):
        """ pre: x and lists of numeric types, len(x)==len(y),
                 Name - string,
                 colour in PenColours,
                 style in PenStyles
            post: Name can be included in items to Plot()
        """
        # http://www.wxpython.org/docs/api/wx.lib.plot.PolyLine-class.html
        xy = map(lambda a, b:(a, b), x, y)
        pl = plot.PolyLine(xy,  colour=cdb.Find(colour),  legend=Name,  width=width,  style=PenStyleDic[style])
        self.PlotItems[Name]  = pl



    def AddMarkerPlotItem(self, x, y, Name, colour="BLACK",  style="SOLID", marker="circle",  width=2,  size=2 ):
        """ pre: As MakeLinePlotItem and marker in MarkerShapes
        post:  Name can be included in items to Plot() """

        xy = map(lambda a, b:(a, b), x, y)
        pl = plot.PolyMarker(xy, colour=cdb.Find(colour), legend=Name,
                             fillstyle=PenStyleDic[style], size=size,
                             width=width,  marker=marker)
        self.PlotItems[Name]  = pl


    def Plot(self, items,  title='', xLabel='', yLabel='' ):
        """ pre: self.HasItem(item) for all items,
                 title, xLabel, yLabel - string
            post: items are plotted """
        if type(items) == types.StringType:
            items = [items]
        gfx = map(self.PlotItems.__getitem__, items)
        pg = plot.PlotGraphics((gfx), title=title, xLabel=xLabel, yLabel=yLabel)
        self.client.Draw(pg)


    def OnFileClose(self, *args):
        self.Close()

    def CreateMenu(self):
        self.mainmenu = wx.MenuBar()

        menu = wx.Menu()
        menu.Append(200, 'Page Setup...', 'Setup the printer page')
        self.Bind(wx.EVT_MENU, self.OnFilePageSetup, id=200)

        menu.Append(201, 'Print Preview...', 'Show the current plot on page')
        self.Bind(wx.EVT_MENU, self.OnFilePrintPreview, id=201)

        menu.Append(202, 'Print...', 'Print the current plot')
        self.Bind(wx.EVT_MENU, self.OnFilePrint, id=202)

        menu.Append(203, 'Save Plot...', 'Save current plot')
        self.Bind(wx.EVT_MENU, self.OnSaveFile, id=203)

        menu.Append(205, 'Close', 'Enough of this already!')
        self.Bind(wx.EVT_MENU, self.OnFileClose, id=205)
        self.mainmenu.Append(menu, '&File')


        menu = wx.Menu()
        menu.Append(211, '&Redraw', 'Redraw plots')
        self.Bind(wx.EVT_MENU,self.OnPlotRedraw, id=211)
        menu.Append(212, '&Clear', 'Clear canvas')
        self.Bind(wx.EVT_MENU,self.OnPlotClear, id=212)
        menu.Append(213, '&Scale', 'Scale canvas')
        self.Bind(wx.EVT_MENU,self.OnPlotScale, id=213)
        menu.Append(214, 'Enable &Zoom', 'Enable Mouse Zoom', kind=wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU,self.OnEnableZoom, id=214)
        menu.Append(215, 'Enable &Grid', 'Turn on Grid', kind=wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU,self.OnEnableGrid, id=215)
        menu.Append(217, 'Enable &Drag', 'Activates dragging mode', kind=wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU,self.OnEnableDrag, id=217)
        menu.Append(220, 'Enable &Legend', 'Turn on Legend', kind=wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU,self.OnEnableLegend, id=220)
        menu.Append(222, 'Enable &Point Label', 'Show Closest Point', kind=wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU,self.OnEnablePointLabel, id=222)

        menu.Append(225, 'Scroll Up 1', 'Move View Up 1 Unit')
        self.Bind(wx.EVT_MENU,self.OnScrUp, id=225)
        menu.Append(230, 'Scroll Rt 2', 'Move View Right 2 Units')
        self.Bind(wx.EVT_MENU,self.OnScrRt, id=230)
        menu.Append(235, '&Plot Reset', 'Reset to original plot')
        self.Bind(wx.EVT_MENU,self.OnReset, id=235)

        self.mainmenu.Append(menu, '&Plot')

        menu = wx.Menu()
        menu.Append(300, '&About', 'About this thing...')
        self.Bind(wx.EVT_MENU, self.OnHelpAbout, id=300)
        self.mainmenu.Append(menu, '&Help')

        self.SetMenuBar(self.mainmenu)



